%--------------------------------------------
% Function onoffdiff
% Input: onewave - boolean matrix of size n_neurons x t_wave x 2 where
%                  onewave(i,t,1)=1 if neuron i is in upstate at time
%                  t and onewave(i,t,2)=1 if column i is in downstate at
%                  time t (note: there are times when neuron is not in up or
%                  down state). 
% Outputs: onsets - time for each neuron to transition from down to up
%          onsetdiffs - vector of differences between each onset and
%                       the first neuron to transition
%--------------------------------------------

function [diff_on, diff_off] = onoffdiff(wave_up, wave_dn, mintimeon)
    %%
    plots = false;
    
    wave_d= wave_up-wave_dn;
    
    t_wave = size(wave_up,2);
    diff_on = nan(size(wave_up,1),1);
    diff_off = nan(size(wave_dn,1),1);
    
    idx_act = find(sum(wave_up,2) > mintimeon);
    wave_up_on = wave_up(idx_act, :); %remove if it doesn't meet acitvity min
    wave_dn_on = wave_dn(idx_act, :); 
    
    n=size(wave_up_on,1);
    
    pre=mintimeon; % minimum time down (up) before up (down)
    edge = (-pre:mintimeon-1)>=0; % transition between down and up
    
    onsets = nan(n,1);
    offsets = nan(n,1);
    %
    for i = 1:n % for each neuron
        %%
        i_neuron_up = wave_up_on(i,:);
        onset = strfind(i_neuron_up,edge); % find transition index
        if ~isempty(onset)
            onset = pre+onset(1);
            onsets(i) =  onset;
            i_neuron_dn = wave_dn_on(i,onset:end); % same neuron after onset
            offset = strfind(i_neuron_dn,edge);
            if ~isempty(offset)
                offsets(i) = (onset-1) + pre + offset(1);
            end
        end
    end
    
    % Remove outliers
    % isoutlier(): Outliers are defined as elements more than three 
    % scaled MAD from the median. The scaled MAD is defined as 
    % c*median(abs(A-median(A))), where c=-1/(sqrt(2)*erfcinv(3/2)).
    
    idx_ol_on = isoutlier(onsets,'median'); % oulier indices
    onsets_no_ol = onsets(~idx_ol_on);
    idx_ol_off = isoutlier(offsets,'median');
    offsets_no_ol = offsets(~idx_ol_off);
    
    % Find first onset/offset and relative differences
    onset0=min(onsets_no_ol);
    diff_on(idx_act(~idx_ol_on))=onsets_no_ol - onset0;
    offset0=min(offsets_no_ol);
    diff_off(idx_act(~idx_ol_off))=offsets_no_ol - offset0;
    
    % Diagnostic plot
    if plots
        idx_num_on = ~isnan(onsets);
        idx_num_off = ~isnan(offsets);
    
        figure
        spr=2; % sub plot rows
        spc=5; % sub plot columns
        
        binwidth=50;
        ymax=0.09;
    
        % Mark onsets and offsets on heatmap
        subplot(spr,spc,[1,spc+1])
        imagesc(wave_d);hold on
        scatter(flip(onsets(idx_num_on)),flip(idx_act(idx_num_on)),10,'filled','g');
        scatter(flip(offsets(idx_num_off)),flip(idx_act(idx_num_off)),10,'filled','r');
        axis([0 t_wave 2000 2100]);
        
        % Onsets
        subplot(spr,spc,2);
        h1=histogram(onsets(~isnan(onsets)));
        h1.Normalization = 'probability';
        h1.BinWidth = binwidth;
        axis([0 t_wave 0 ymax]);
        title('onsets no nan');
    
        
        subplot(spr,spc,3); 
        h2=histogram(onsets_no_ol(~isnan(onsets_no_ol)));
        h2.Normalization = 'probability';
        h2.BinWidth = binwidth;
        axis([0 t_wave 0 ymax]);
        title('onset no outliers');
        
        subplot(spr,spc,4);
        h3=histogram(diff_on(~isnan(diff_on)));
        h3.Normalization = 'probability';
        h3.BinWidth = binwidth;
        axis([0 t_wave 0 ymax]);
        title('onset - min(onset)');
        
        subplot(spr,spc,5);
        b=boxchart(onsets);
        
        % Offsets
        subplot(spr,spc,spc+2);
        h1=histogram(offsets(~isnan(offsets)));
        h1.Normalization = 'probability';
        h1.BinWidth = binwidth;
        axis([0 t_wave 0 ymax]);
        title('offset no nan');
        
        subplot(spr,spc,spc+3);
        h2=histogram(offsets_no_ol(~isnan(offsets_no_ol)));
        h2.Normalization = 'probability';
        h2.BinWidth = binwidth;
        axis([0 t_wave 0 ymax]);
        title('offset no outliers');
        
        subplot(spr,spc,spc+4);
        h3=histogram(diff_off(~isnan(diff_off)));
        h3.Normalization = 'probability';
        h3.BinWidth = binwidth;
        axis([0 t_wave 0 ymax]);
        title('offset - min(offset)');
        
        subplot(spr,spc,spc+5);
        b=boxchart(offsets);

        drawnow;
        %saveas(gcf, ['./figures/testfig.png']);

    end
    end % end function onsetdiffs
    