%-------------------------------------------------------------------------------------
% latencymap.m
% Plots latency maps (onset differences) in 3D brain and produces similarity plots
% 
% Requires 
% - files: ico5_pial_surf_lh.mat, parcels.csv, so_*folder*.mat (for each folder)
% - functions: mysavefig() (optional, for pdfs)
% - output folder: 'figures'
%
% Last revised: Aug 4, 2023
%-------------------------------------------------------------------------------------
% cd /bazhlab/mnavaszu/meg/matlab/plot_voltages
% matlab -nodisplay -nosplash -nodesktop -r "run('latencymap.m');exit;"

%-------------------------------------------------------------------------------------
%% Setup
%-------------------------------------------------------------------------------------
close all
font_size=14;
set(0,'defaultAxesFontSize',font_size,...
    'defaultAxesTitleFontSizeMultiplier', 1,...
    'defaultLineLineWidth',1);

% Add path for general files and functions
addpath('/bazhlab/mnavaszu/meg/matlab/files',...
'/bazhlab/mnavaszu/meg/matlab/functions');

%%
loaddata = 1; % bool, load data from ./files folder

%-------------------------------------------------------------------------------------
%% Load files
%-------------------------------------------------------------------------------------

%% load ico5 surfaces and parcel identities
disp('Loading ico...');
srf = struct;
%matfile('/bazhlab/ysokolov/meg/newConn/surfCoord/');
mObj = matfile('ico5_pial_surf_lh.mat');
srf = mObj.srf;
disp('done!')

% Load parcels
disp('Loading parcels...');
parc = load('parcels.csv'); % Load parcels (0 = wall)

% Find idx of medial wall & not medial wall
idx = (1:10242);
idx_ml = find(parc == 0);
idx_nml = setdiff(idx, idx_ml);
disp('done!')

%-------------------------------------------------------------------------------------
%% Select source folders
%-------------------------------------------------------------------------------------
if 1 % only individual sims to appear in paper
% folders={'SO_base',...
% 'r0_0025',...
% %'r0_005', ...
% 'p01',...
% %'p05',...
% %'low_allCX_3x',...
% 'low_01_5x'...
% };
folders={'SO_base',...
'r0_0025',...
'p01',...
'low_01_5x'...
};
end

%folders = {'low_005_5x'};
n_fld = length(folders);

for i_fld=1:n_fld
    close all
    folder = folders{i_fld};
    %-------------------------------------------------------------------------------------
    %% Load data
    %-------------------------------------------------------------------------------------
    if loaddata
        disp(['Loading data ' folder ' (' num2str(i_fld) '/' num2str(n_fld) ') ...']); tic
        load(['./files/so_prop/so_' folder '.mat']);
        disp('done'); toc
    end
    %% Make sure size is right
    n=10242;
    n_nml = numel(idx_nml);
    n_onset_diffs = numel(onset_diffs);
    assert(mod(n_onset_diffs,n)==0, ['Size of onset_diffs ' num2str(n_onset_diffs) ' must be a multiple of 10242']);
    
    notnancols=find(~all(isnan(onset_diffs),1)); % omit nan up-states
    onset_diffs = onset_diffs(:,notnancols);

    n_up = size(onset_diffs,2); % number of upstates
    
    onset_diffs_rmnan=onset_diffs;
    onset_diffs_rmnan(isnan(onset_diffs_rmnan))=max(onset_diffs,[],'all')*10; %dummy huge onset for nans
    %onset_diffs=reshape(onset_diffs,[n, n_up]);
    
    %-------------------------------------------------------------------------------------
    %% Similarity
    %-------------------------------------------------------------------------------------
    if 0
        similarity = nan(n_up, n_up);
        
        onset_diffs_norm = onset_diffs./max(onset_diffs);
        
        for i_up = 1:n_up
            for j_up = 1:i_up
                
                % Get point-by-point absolute difference between normalized latencies
                ij_pdiff=abs(onset_diffs_norm(:,i_up)-onset_diffs_norm(:,j_up));
                
                % Average pointwise differences over all points
                ij_pdiff_mean=mean(ij_pdiff, 'omitnan');
                assert(numel(ij_pdiff_mean)==1);
                
                % Get inverse
                similarity(i_up,j_up) = 1-ij_pdiff_mean;
                similarity(j_up,i_up) = similarity(i_up,j_up);
            end
        end
        %
        up_dist=repelem(0:n_up-1, n_up, 1)-repelem(0:n_up-1, n_up,1)';
        
        similarity_v = reshape(similarity, 1, numel(similarity));
        up_dist_v=reshape(up_dist,1,numel(up_dist));
        %
        tb = table(up_dist_v', similarity_v','VariableNames',["Dist","Similarity"]);
        G = groupsummary(tb, "Dist", "mean");
    end
    %-------------------------------------------------------------------------------------
    %% Plot
    %-------------------------------------------------------------------------------------
    disp('Plotting...');
    close all
    %figure
    n_col = n_up+1;
    n_row = 2*ceil(n_up/n_col);
    tl = tiledlayout(n_row,n_col, 'Padding','loose', 'TileSpacing','tight');

    brain_w_cm=4;
    brain_h_cm=brain_w_cm*.8;
    s_w = 1/n_col;
    s_h = 1/n_row;
    set(gcf,'units','centimeters','position',[0 0 n_col*brain_w_cm n_row*brain_h_cm])
    %
    c_quant = quantile(onset_diffs(~isnan(onset_diffs)),[0 .5 1],"all");
    c_ticks = unique(round(c_quant,-2));
    c_lims =c_ticks([1,end]); % onset delay limit (ms)
    v_lims = [-80 -50]; % voltage limits (mV)
    
    % Loop thorugh upstates

    
    for i=1:n_up
        
        i_s1 = i+floor((i-1)/n_col)*n_col; % index for upper subplot 
        i_s2 = i_s1+n_col; % index for lower subplot
        %ax_x = (i-1)*s_w+s_w*0.05;
        %ax_w = s_w*0.9;
        %disp([i ax_x])
        
        %s1=subplot(n_row,n_col,i_s1);
        s1=nexttile(i_s1);
        tH = triplot(srf.pial.vertices,srf.pial.faces,onset_diffs_rmnan(:,i));
        view(-90,90);
        camtarget([-15 -18 400]);
        axis vis3d 
        %campos([0 0 100]);
        
        tH.FaceColor='Flat';
        camlight headlight;material dull;lighting gouraud;
        colormap(s1,hot);
        clim(c_lims);
        %axis tight
        
        t=title(['Upstate ' num2str(i)],'Units', 'normalized');
        
        %s1.Position([1,3]) = [ax_x, ax_w];
        
        %s2=subplot(n_row,n_col,i_s2);
        s2=nexttile(i_s2);
        %%
        tH = triplot(srf.pial.vertices,srf.pial.faces,onset_diffs_rmnan(:,i));
        view(-90,0);
        tH.FaceColor='Flat';
        camlight headlight;material dull;lighting gouraud;
        colormap(s2,hot);
        clim(c_lims);
        %s2.Position([1,3]) = [ax_x, ax_w];
        %axis tight
        %%
        set([s1;s2], 'Colorscale', 'log');
        axis([s1,s2], 'tight');
        % Print what percentage of neurons participate in upstate
        prc_part = round(numel(find(~isnan(onset_diffs(:,i))))/n_nml*100,1);
        txt = ['active: ' num2str(prc_part) '%'];
        annotation('textbox',...
            [s2.Position(1), s2.Position(2), 0,0],...
            'String', txt, 'EdgeColor', 'none', 'FitBoxToText', 'on', 'FontSize', font_size);
    drawnow
    end
    %
    cb2=colorbar(s2);%,'location', 'eastoutside');
    cb2.Ticks = c_ticks;
    cb2.TickLabels = num2str(c_ticks/1000);
    cb2.Title.String={'Onset';'delay';'[s]'};
    %cb2.Layout.Tile='east';
    cb2.Position([1,2,4]) = [s2.Position(1)+s2.Position(3)*1.5, .1,0.35];
    %
    %
    
%     txt = ['Upstate #:'];
%     annotation('textbox',...
%         [s_w/4, 0.95, 0,0],...
%         'String', txt, 'EdgeColor', 'none', 'FitBoxToText','on');
%     txt = ['% active neurons:'];
%     annotation('textbox',...
%         [s_w/8, s2.Position(2), 0, 0],...
%         'String', txt, 'EdgeColor', 'none','FitBoxToText','on');
    
    pth_sv='./figures/latency'; % path to folder for saving figure
    
    figname = ['lmap_' folder];
    disp('done!')
    
    
    mysavefig(gcf, pth_sv, figname, font_size);
    
    try
        savefig(gcf,[pth_sv '/' figname] )
    catch
        disp('Could not save .fig file');
    end
    % %%
    % %saveas(gcf, [pth_sv '/' figname '.png']);
    % 
    %  figure
    %  scatter(up_dist_v,similarity_v); hold on;
    %  plot(G.Dist, G.mean_Similarity, '*-');
    %  ylabel('Similarity')
    %  xlabel('Cycle difference')
    %  figname = ['simil_' folder];
    % 
    %mysavefig(gcf, pth_sv, figname);
     %saveas(gcf, [pth_sv '/' figname '.png']);
    % 
    disp(['done with folder ' folder '! (' num2str(i_fld) '/' num2str(n_fld) ')']);
    
end % end folders loop