%-------------------------------------------------------------------------------------
% lfps.m
% Plots voltage activity for (spatially) local clusters of neurons in the
% cortex (L2)
%
% Requires:
% - files: time_cx_*folder* (for each folder)
% - scripts: sampleparcels.m
% - output folder: 'figures'
%
% Last revised: Aug 5, 2023
%-------------------------------------------------------------------------------------
% cd /bazhlab/mnavaszu/meg/matlab/plot_voltages
% matlab -nodisplay -nosplash -nodesktop -r "run('lfps_singlecell.m');exit;"


%-------------------------------------------------------------------------------------
%% Setup
%-------------------------------------------------------------------------------------
import = 0;
loaddata = 1;
figsonly =0;
font_size = 12;
set(0,'defaultAxesFontSize',font_size,...
    'defaultAxesTitleFontSizeMultiplier', 1,...
    'defaultLineLineWidth',1);
%close all
% Add paths for general files and functions
addpath('/bazhlab/mnavaszu/meg/matlab/files',... % general files
    '/bazhlab/mnavaszu/meg/matlab/functions',...    % general functions
    '/bazhlab/mnavaszu/meg/matlab/plot_voltages/files',... % specific files
    '/bazhlab/mnavaszu/meg/matlab/plot_voltages/functions'... % specific functions
    );

%-------------------------------------------------------------------------------------
%% Load clusters
%-------------------------------------------------------------------------------------
%ngroups = 5;
%ngroups_ml = ngroups+1;
if ~figsonly
    disp('Sampling parcels...');
    %ClusterNeurons;
    % Load parcels
    parcels = load('parcels.csv'); % Load parcels (0 = wall)

    % Find idx of medial wall & not medial wall
    idx = (1:10242);
    idx_ml = find(parcels == 0);
    idx_nml = setdiff(idx, idx_ml);
    %%
    % Use 10 regions from parcels sampled from 10 functional networks
    sampleparcels;

    % Use 10 functional networks directly
    % idx_col = idx_nwk;

    % Use random clustering algorithm
    % idx_clusters= csvread('10clusters_nml.csv');
    % idx_col = idx_clusters+1;
    % idx_col(idx_col==11)=0;
    %ngroups_ml = ngroups+1;

end

%-------------------------------------------------------------------------------------
%% Loop through folders
%-------------------------------------------------------------------------------------
%% Folder of origin
% 247 - Full connectivity and probability
% > Radius change
% 248 - R = 0.05, P = 1
% n3_1 - R = 0.01, P = 1
% n3_2 - R = 0.0025, P = 1
% > Probability change
% 252 - R = 10, P = 0.3
% n3_3 - R = 10, P = 0.1
% n3_4 - R = 10, P = 0.5
% R, p, d, ud
%folders = {'SO_base','r0_0025','r0_005', 'p01','unsc_delay_25', 'delay_1000', 'weak_cx_5x', 'low_longrange_5x', '247_half_long5x'};
folders = {'low_01_5x','r0_0025','p01'};
% sims = {'low_01_5x/out_high1_v2', 'low_01_5x/out_high2', ...
%     'low_01_5x/out_high3', 'low_01_5x/out_high4',...
%     'low_01_5x/out_low1', 'low_01_5x/out_low2', ...
%     'low_01_5x/out_low3', 'low_01_5x/out_low4'};
% folders = {'low_01_5x_high10', 'low_01_5x_high25', ...
%     'low_01_5x_high05', 'low_01_5x_high50',...
%     'low_01_5x_low10', 'low_01_5x_low25', ...
%     'low_01_5x_low05', 'low_01_5x_low50'};
n_fld = length(folders);
%%
for i_fld=1:n_fld
    if ~figsonly
        folder = folders{i_fld}; %folder name string
%-------------------------------------------------------------------------------------
        %% Import/load data
        %-------------------------------------------------------------------------------------
        if import
            %% Import data
            fname=[pth folder '/out/time_cx'];
            fname=[pth folder '/time_cx'];
            disp(['Importing data ' folder ' (' num2str(i_fld) '/' num2str(n_fld) ') ...']); tic
            time_cx = importdata(fname);
            disp('done'); toc
            disp(['Original size of voltage vector: ' num2str(size(time_cx))]);
            
            %% Crop data
            disp('Cropping and saving...'); tic
            time_cx = time_cx(end-td+1:end, 2:end)'; % remove 1st neuron (counter) and crop time
            save(['./files/time_cx_'  folder  '.mat'],'time_cx','-v7.3');
            disp('done'); toc
        elseif load_data
            disp(['Loading data ' folder ' (' num2str(i_fld) '/' num2str(n_fld) ') ...']); tic
            load(['./files/time_cx_'  folder  '.mat']);
            if load_soprops
                load(['./files/so_prop/so_' folder '.mat']);
            end
            disp('done'); toc
        end
        %% Load voltage time series
        if loaddata
            disp(['Loading data ' folder '...']); tic
            load(['./files/time_cx_' folder '.mat']);
            load(['./files/so_prop/so_' folder '.mat']);

            disp('done'); toc
            td = size(time_cx,2);
            x = linspace(0,td,td); % create time vector
        end
        %% Sort neurons into clusters

        %[s, idx_sorted] = sort(idx_col);
        %time_cx_sorted = time_cx(idx_sorted, :);

        %% Get means

        %% Remove medial wall for mean and analysis
        time_cx_nml = time_cx(idx_nml,:);

        %% Find average

        time_cx_mean = mean(time_cx_nml);


        cl_means = nan(ngroups,td);
        % for i= 1:ngroups
        %     idx = idx_col==i;
        %     cl_means(i,:) = mean(time_cx(idx, :));
        % end
        for i= 1:ngroups
            idx = idx_col==i;
            cl_means(i,:) = mean(time_cx(idx, :));
        end
        % Slightly smooth cluster mean signals with moving average of 20ms
        % window (20 points)
        cl_means = movmean(cl_means,20,2);
    end
   %% Plots
close all
f=figure('Units', 'centimeters', 'Color', 'w');
f.Position(3:4)=[18,18];

% Axis limits and labels
ymin=-85;
ymax=-50;
yrange = ymax-ymin;
yt=[ymin ymax];
ytl={'-85mV', '-50mV'};
thr_up = -65;

xt=0:10000:30000;
xtl=num2str(xt'/1000);
xtls=xtl;
%xtls=[xtl repelem('s',length(xtl))'];

% Find colorblind-friendly scheme
% https://colorbrewer2.org/?type=diverging&scheme=PuOr&n=11
% clcol = [
%     250, 250, 250;
%         171,217,233;
%         116,173,209;
%         69,117,180;
%         49,54,149;
%         0,0,0;
%         165,0,38;
%         215,48,39;
%         244,109,67;
%         253,174,97;
%         %254,224,144;
%         254,210,0;
% ]/255;
clcol = [
    255, 255, 255;
        171,217,233;
        116,173,209;
        69,117,180;
        49,54,149;
        0,0,0;
        165,0,38;
        215,48,39;
        244,109,67;
        253,174,97;
        %254,224,144;
        254,210,0;
    0,255,0
    ]/255;


% Subplot properties
n_b = 3; % # subplot spaces to plot brain
n_sp = ngroups + n_b;

% Plot clusters
disp('Plotting clusters...');
%tl=subaxis(n_sp,3, 'SpacingVert',0.1);
mrg = .07; %margin
s1=subaxis(n_sp,3,[1 4 7], 'SpacingVert',0.012,'SpacingHoriz',.03,...
    'MR',mrg,'ML',mrg,'MT',.5*mrg,'MB',mrg);
%%
tH = triplot(srf.pial.vertices,srf.pial.faces,idx_brd);
view(-90,0)
tH.FaceColor='Flat';
camlight headlight;material dull;lighting gouraud;
s1.CameraTarget(3)=0;
colormap(s1,clcol);
%colormap(gca,clcol);
cb=colorbar('eastoutside');
%
cb.Limits = [1,ngroups];
cb.Ticks = 1.5:.9:ngroups+1;
cb.TickLabels = num2str([1:ngroups]');
cb.TickLength = 0;
set( cb, 'YDir', 'reverse' );

axis tight

%% Plot mean voltage
disp('Plotting mean voltage...');
%s02 = subaxis(2);
%text(0,0.1,'Layer average voltage'); axis off
%
s2=subaxis( 5);
plot(x,time_cx_mean);hold on
plot(locs_peaks, peaks+5, 'vr', 'MarkerFaceColor','r', 'MarkerSize', 2);
plot(locs_mins, mins-5, '^k','MarkerFaceColor','k','MarkerSize', 2);
ylim([ymin, ymax]);
title('Layer average voltage');

axis off
%set(s2.XAxis,'visible', 'on');
%set(s2, 'XTick',xt, 'XTickLabel',xtls)

%% Plot all voltages
disp('Plotting cluster voltages...');
% Titles
% s7=subaxis((n_b-1)*3+1);
% text(0,0.1,'Single-cell voltage'); axis off
% s8=subaxis( (n_b-1)*3+2);
% text(0,0.1,'LFP'); axis off
% s9=subaxis( (n_b-1)*3+3);
% text(0,0.1,'Local population voltages'); axis off
%%
for i=1:ngroups
    %%
    i_sp = (n_b+i-1)*3;
    idx = idx_col==(i);
    time_cx_cl = time_cx(idx, :);

    act_cl=sum(time_cx_cl>thr_up,2); %active neurons in cluster
    c = find(act_cl>1000,2);
    if isempty(c)
        c = [1,2];
    end

    % Plot single-cell voltage
    s31=subaxis(i_sp+1);
    p=plot(x,time_cx_cl(c,:)); % plot 2 columns form each cluster
    %
    p(1).Color(4) = 1;
    p(2).Color = [.5 .5 .5 .45];%.68;
    %p(2).LineWidth = 2;
    %
    ylim([ymin, ymax])

    axis off

    %% Plot cluster average voltages
    s32=subaxis(i_sp+2);
    plot(x,cl_means(i,:), 'Linewidth', 1);
    ylim([ymin, ymax])

    axis off
    %set(s32.YAxis,'visible', 'on');
    %set(s32,'YTick',yt, 'YTickLabel', ytl)

    %% Plot heatmap of voltage
    s33=subaxis(i_sp+3);
    h= imagesc(time_cx_cl, [-70,-50] );
    colormap(s31,parula);

    axis off

    % if i==floor(ngroups/2)
    %     s33.YLabel.Visible='on';
    %     set(s33,'YAxisLocation', 'right');
    %     ylabel(s33,'Neurons');
    % end

    if i==1
        %% Set titles
        title(s31,'Single-cell voltage');
        title(s32, 'LFP');
        title(s33, 'Local region voltages');

        %% Fix voltage colorbar
        cb2=colorbar('north');
        cb2.Position = [s33.Position(1)+s33.Position(3)*.75, s33.Position(2)+s33.Position(4)*1.8, s33.Position(3)*.25, 0.015];
        cb2.Title.String = 'mV';
    end
    %tl.TileSpacing='tight';

end
disp('Final formatting...');
% Include one axis for scale

set(s31.XAxis,'visible', 'on');
set(s32.XAxis,'visible', 'on');
set(s33.XAxis,'visible', 'on');

set([s31, s32, s33], 'XTick',xt, 'XTickLabel',xtls);% Set Tick Labels

xlabel(s32,'Time (s)')

% Include scale for y axis
s03=subaxis(6);
ylim([ymin, ymax])
xlim([0 x(end)]);
axis off;
scalesize=20;
scale=annotation('rectangle');
scale.Parent = s03;
scale.Position = [0, ymin, 0, scalesize] ;
t=text(0,1,[num2str(scalesize) 'mV']);
t.Parent = s03;
%

t.Position = [xt(2)/4, ymin+scalesize/2] ;

%% Fix cluster colorbar
cb.Position = [s31.Position(1)*0.5, s31.Position(2)-0.01, 0.02, s1.Position(2)-s31.Position(2)];

    %% Save figure
    %saveas(gcf, ['./figures/lfps/lfp_sc_' folder '_' num2str(ngroups) 'cl.png']);

    mysavefig(gcf, './figures/lfps_v2/',['lfp_sc_' folder ], font_size);

    disp(['done with folder ' folder '! (' num2str(i_fld) '/' num2str(n_fld) ')']);

end