%--------------------------------------------------------------------------
% make_video.m
% Creates a video from the voltage readings of a simulation 
% Displays voltage of columns in layer 2 on 3D ico surface
% and real time evolution of layer average voltage
%---------------------------------------------------------------------------

% cd /bazhlab/mnavaszu/meg/matlab/video && matlab -nodisplay -nosplash -nodesktop -r "run('make_video.m');exit;"


%-------------------------------------------------------------------------------------
%% Setup
%-------------------------------------------------------------------------------------
close all
font_size = 12;
set(0,'defaultAxesFontSize',font_size,...
    'defaultAxesTitleFontSizeMultiplier', 1,...
    'defaultLineLineWidth',.8);
% Add paths for general files and functions
addpath('/bazhlab/mnavaszu/meg/matlab/files',... % general files
'/bazhlab/mnavaszu/meg/matlab/functions' ...   % general functions
);
% Options
import = 1; % import directly from sim output?
loaddata = 0; % load pre-saved cropped data?


%-------------------------------------------------------------------------------------
%% Select source folders
%-------------------------------------------------------------------------------------
pth = '/bazhlab/mnavaszu/meg/sims/stim/10mm5x/';
outfld = './videos/stim'; % where to save video file
sim = '10mm5x'; % main sim where the data is coming from

if ~isfolder(outfld);
    mkdir outfld;
end

folders = {'out_c1'};
n_f = length(folders);

if loaddata | import
    % Load parcels
parc = load('parcels.csv'); % Load parcels (0 = wall)

% Find idx of medial wall & not medial wall
idx = (1:10242);
idx_ml = find(parc == 0);
idx_nml = setdiff(idx, idx_ml);
    %% load ico5 surfaces and simulation data
    srf = struct;
    mObj = matfile('ico5_pial_surf_lh.mat');
    srf = mObj.srf;
end

%-------------------------------------------------------------------------------------
%% Loop through folders
%-------------------------------------------------------------------------------------

for i_f = 1:n_f
    
      folder = folders{i_f};

      %-------------------------------------------------------------------------------------
      %% Import/load data
      %-------------------------------------------------------------------------------------
      if import
          %% Import data
          % Spindles: 206, n2_5, n2_4, (227?) ( /rand6
          td = 15000; %time duration (ms)
          disp(['Importing data (' folder ')...']);
          data =importdata([pth, folder, '/time_cx']);
          data = data(end-td+1:end,2:end);
          disp('... done importing data');
      elseif loaddata
          %% Load data
          disp(['Loading data ' folder ' (' num2str(i_f) '/' num2str(n_f) ') ...']); tic
          data=load(['/bazhlab/mnavaszu/meg/plot_voltages/files/time_cx_' folder '.mat'], 'time_cx');
          data=data.time_cx';
          disp('done'); toc
      end
      tmax = size(data,1);
      ncol = size(data,2);
      
      %-------------------------------------------------------------------------------------
      %% Average voltage
      %-------------------------------------------------------------------------------------
      
      %% Remove medial wall for mean and analysis
      data_nml = data(:,idx_nml);
      
      % Find average voltage
      data_mean = mean(data_nml,2);
      assert(length(data_mean)==tmax);
      
      %-------------------------------------------------------------------------------------
      %% Setup frames and open video file
      %-------------------------------------------------------------------------------------
      
      % Separate into frames of 1/20 second
      win = 50;
      num_frames = tmax/win;
      data_chunked = reshape(data,[win,num_frames,ncol]);
      
      ptimes = 1:win:tmax;
      vid_name = [outfld '/' sim '_' folder];
      
      % Choose singel cells to plot
      c1_idx=1060;
      c1=data(:,c1_idx);
      c1(c1>-50)=max(c1); % fix spikes truncated by low resolution sampling

      % Set axis limits
      v_lims=[-70 -55]; % lfp y limits - Slow wave range
      %[-80 -50]; % lfp y limits - Slow wave
      %[-69 -67]; % lfp y limits - Spindles
      c_lims=[-80 45]; % single cell y limits
      
      % Set ticks
      xt=0:5000:tmax;
      xtl=num2str(xt'/1000);

      %-------------------------------------------------------------------------------------
      %% Write video file
      %-------------------------------------------------------------------------------------
      
      disp('Creating video...')
      video = VideoWriter(vid_name); %create the video object
      video.FrameRate = 1000/win;
      open(video); %open the file for writing
      f=figure('Units', 'centimeters', 'Color', 'w');
      f.Position(3:4)=[15,15];

      for i=1:num_frames %where num_frames is the number of images
          
          if mod(i,100)==0 % report progress sometimes
              disp(['Frame ' num2str(i) '/' num2str(num_frames)]);
          end
          
          data_frame = squeeze(data_chunked(:,i,1:ncol));
          I = mean(data_frame',2);
          
          clf;set(gcf,'color','w')
          tl = tiledlayout(6,1, 'TileSpacing', 'tight', 'Padding', 'compact');
          
          s1=nexttile([4,1]);
        
          tH = triplot(srf.pial.vertices,srf.pial.faces,I);
          view(-90,0)
          tH.FaceColor='Flat';
          camlight headlight;material dull;lighting gouraud; 
          
          axis tight

          %% Fix voltage colorbar
          cb=colorbar('south');
          cb.Position = [s1.Position(1)+s1.Position(3)*.75, s1.Position(2), s1.Position(3)*.25, 0.025];
          cb.Title.String = 'mV';
          caxis(v_lims);
          

          nexttile
          plot(ptimes(i):tmax,data_mean(ptimes(i):end),'k');
          xlim([0 tmax]);
          hold on;
          plot(1:ptimes(i),data_mean(ptimes(1):ptimes(i)),'Color', [1 1 1]*.6);
          xline(ptimes(i), 'r', 'Linewidth', 2);
          plot(ptimes(i), v_lims(2), 'v', 'MarkerSize', 7, 'MarkerFaceColor','r');

          hold off;
          ylabel('Voltage (mV)');
          title('LFP');
          %xlabel('Time (s)')
          ylim(v_lims);yticks(v_lims);
          set(gca().XAxis,'visible', 'off');
          box off

          nexttile
          plot(ptimes(i):tmax,c1(ptimes(i):end),'k');
          xlim([0 tmax]);
          hold on;
          plot(1:ptimes(i),c1(ptimes(1):ptimes(i)),'Color', [1 1 1]*.6);
          xline(ptimes(i), 'r', 'Linewidth', 2);
          plot(ptimes(i), c_lims(2), 'v', 'MarkerSize', 7, 'MarkerFaceColor', 'r');
          hold off;
          %ylabel('Voltage (mV)');
          xlabel('Time (s)')
          ylim(c_lims);yticks(c_lims);
          title('Single-cell voltage');
          set(gca, 'XTick',xt, 'XTickLabel',xtl, 'TickDir', 'out');% Set Tick Labels

          box off

          frame = getframe(gcf);
          %saveas(gcf, './videos/testfig.png')
          writeVideo(video,frame); %write the image to file
      end % end frame loop
      
      close(video);
      close all

end %end folders loop