%-------------------------------------------------------------------------------------
% plot_voltages_noamp.m
% Generates a figure with
% 1) Voltage of CX L2 neurons over time
% 2) Average voltage of all CX L2 voltages over time (with no x or y axes)
% 3) Voltage of two individual neurons in CX L2
% 3) Histogram of voltages of all CX L2 neurons
% 4) Histograms of onsets and offsets
% 
% Requires 
% - files: ico5_pial_surf_lh.mat, parcels.csv, 
%          time_cx_*folder*.mat (if loading) or time_cx (if importing) for each folder
% - functions: clean_onoff(), onoffdiff(), mysavefig() (optional, for pdfs)
% - output: folders 'files' and 'figures'
%
% Last revised: Apr. 25, 2024
%-------------------------------------------------------------------------------------
% cd /bazhlab/mnavaszu/meg/matlab/plot_voltages
% matlab -nodisplay -nosplash -nodesktop -r "run('plot_voltages_noamp.m');exit;"


%-------------------------------------------------------------------------------------
%% Setup
%-------------------------------------------------------------------------------------
close all
fs=11;
set(0,'defaultAxesFontSize',fs,...
    'defaultAxesTitleFontSizeMultiplier', 1,...
    'defaultLineLineWidth',.8);

% Options: import data, do intermediate plots
figsonly=0; % do just figures, nothing else?
import = 0; % bool, import complete data from sim?
load_data = 1; % bool, load saved data from mat file?
load_soprops = 1; % bool, load all processing results intead of processing again?
plots = 0; % do intermediate diagnostic plots?

% Add paths for general files and functions
addpath('/bazhlab/mnavaszu/meg/matlab/files',... % general files
'/bazhlab/mnavaszu/meg/matlab/functions',...    % general functions
'/bazhlab/mnavaszu/meg/matlab/plot_voltages/files',... % specific files
'/bazhlab/mnavaszu/meg/matlab/plot_voltages/functions'... % specific functions
);

% Load parcels
parc = load('parcels.csv'); % Load parcels (0 = wall)

% Find idx of medial wall & not medial wall
idx = (1:10242);
idx_ml = find(parc == 0);
idx_nml = setdiff(idx, idx_ml);
n_nml = numel(idx_nml);

%% load ico5 surfaces and parcel identities
srf = struct;
mObj = matfile('ico5_pial_surf_lh.mat');
srf = mObj.srf;

%-------------------------------------------------------------------------------------
%% Select source folders
%-------------------------------------------------------------------------------------

pth = '/bazhlab/bmarsh/meg/radius_prob/';
%folders = { 'low_longrange_3x', 'low_longrange_4x', 'low_longrange_5x',...
%'247_half_long2x', '247_half_long3x', '247_half_long4x','247_half_long5x',};

% folders = {'delay_01','247_dx2', 'delay_1000', 'delay_1250', 'delay_1500',...
% 'unsc_delay_1','unsc_delay_10', 'unsc_delay_30',...
% 'weak_cx_2x','weak_cx_5x', 'weak_cx_7_5x','weak_cx_8x'};

if 0 % all folders for summary plots
folders={ 'SO_base',...
'r0_0025','r0_004','r0_005', 'r0_01', ...
            'p00562','p01','p01418', 'p03', 'p05', 'p08',...
            'unsc_delay_1','unsc_delay_10', 'unsc_delay_20','unsc_delay_25','unsc_delay_30',...
            'delay_01','delay_10', 'delay_20', 'delay_80', 'delay_1000',...% 'delay_1250', 'delay_1500',...
            'low_allCX_2x','low_allCX_3x', 'low_allCX_4x','low_allCX_5x',...
            'high_allCX_2x','high_allCX_3x','high_allCX_4x', 'high_allCX_5x',...
            'high_longrange_2x','high_longrange_5x',...
            'n3_min0025','n3_min004','n3_min005','n3_min01', 'n3_min1',...
            'low_local_2x', 'low_local_5x',...
            'low_longrange_2x','low_longrange_3x', 'low_longrange_4x', 'low_longrange_5x','low_longrange_10x',...
            '247_halfWeights_base','247_half_long2x', '247_half_long3x', '247_half_long4x','247_half_long5x',...
            'low_05_5x', 'low_025_5x','low_01_5x', 'low_005_5x','low_0025_5x'};
end

if 0 % only individual sims to appear in paper
folders={ %'SO_base',...
'r0_0025',...%'r0_005', ...
            'p01', 'p05',...
            'low_allCX_3x',...
            'low_01_5x'
            };
end

folders = {'low_01_5x'};

n_fld = length(folders);

%-------------------------------------------------------------------------------------
%% Loop through folders
%-------------------------------------------------------------------------------------

for i_fld=1:n_fld
    
    if ~figsonly
        %% Folder of origin
        % 247 - Full connectivity and probability
        % > Radius change
        % 248 - R = 0.05, P = 1
        % n3_1 - R = 0.01, P = 1
        % n3_2 - R = 0.0025, P = 1
        % > Probability change
        % 252 - R = 10, P = 0.3
        % n3_3 - R = 10, P = 0.1
        % n3_4 - R = 10, P = 0.5
        
        folder = folders{i_fld};
        
        %% Define time vector
        
        %t0 = 5000; % cut initial transient of 5000 miliseconds
        td = 30000; % time duration of interest
        %tf = t0+td-1; % take td seconds after t0
        %tf = 50000; % take final time at 50k ms
        x = linspace(0,td,td); % create time vector
        
        %-------------------------------------------------------------------------------------
        %% Import/load data
        %-------------------------------------------------------------------------------------
        if import
            %% Import data
            fname=[pth folder '/out/time_cx'];
            disp(['Importing data ' folder ' (' num2str(i_fld) '/' num2str(n_fld) ') ...']); tic
            time_cx = importdata(fname);
            disp('done'); toc
            disp(['Original size of voltage vector: ' num2str(size(time_cx))]);
            
            %% Crop data
            disp('Cropping and saving...'); tic
            time_cx = time_cx(end-td+1:end, 2:end)'; % remove 1st neuron (counter) and crop time
            save(['./files/time_cx_' folder '.mat'],'time_cx','-v7.3');
            disp('done'); toc
        elseif load_data
            disp(['Loading data ' folder ' (' num2str(i_fld) '/' num2str(n_fld) ') ...']); tic
            load(['./files/time_cx_' folder '.mat']);
            if load_soprops
                load(['./files/so_prop/so_' folder '.mat']);
            end
            disp('done'); toc
        end
        %%
        n = size(time_cx,1); % number of neurons
        disp(['Cropped size of voltage vector: ' num2str(size(time_cx))]);
        
        % Note: time_cx contains nans (perhaps spikes that get too high and cannot be recorded?)
        % numel(time_cx(isnan(time_cx)))
        
        %-------------------------------------------------------------------------------------
        %% Average voltage
        %-------------------------------------------------------------------------------------
        
        %% Remove medial wall for mean and analysis
        time_cx_nml = time_cx(idx_nml,:);
        
        %% Find average
        
        time_cx_mean = mean(time_cx_nml);
        disp(['Size of avg voltage vector: ' num2str(size(time_cx_mean))]);
        
        if ~load_soprops
            %% Period
            prominence = (max(time_cx_mean) - min(time_cx_mean))/2;
            pksep = 500; % minimum peak separation
            
            [pk_bool, pk_prom]= islocalmax(time_cx_mean, ...
                'MinProminence', max(1,prominence),'MinSeparation',pksep);
            peaks = time_cx_mean(pk_bool);
            locs_peaks = find(pk_bool);
            npeaks = numel(peaks);

            % [min_bool, min_prom]= islocalmin(time_cx_mean, ... 
            %     'MaxNumExtrema', npeaks+2,...
            %     'MinSeparation',pksep);
            % mins = time_cx_mean(min_bool);
            % locs_mins = find(min_bool);
            % nmins = numel(mins);
            nmins = npeaks+1;
            mins = zeros(1,nmins);
            locs_mins = zeros(1,nmins);

            t0=1;
            for i_pk =1:npeaks
                segment = time_cx_mean(t0:locs_peaks(i_pk));
                mins(i_pk) = min(segment);
                locs_mins(i_pk) = find(segment==mins(i_pk),1)+t0-1;
                t0=locs_peaks(i_pk);
            end
            segment = time_cx_mean(t0:end);
            mins(end) = min(segment);
            locs_mins(end) = find(segment==mins(end),1)+t0-1;

            interpeak = diff(locs_peaks);
            interpeak_mean = mean(interpeak);
            interpeak_sd = std(interpeak);
            
            %so_density = numel(peaks)/(td/1000);% SO count / second
            so_freq_direct = 1/(interpeak_mean/1000);% 1/period (in seconds)
            
            so_freq = 1./interpeak;
            so_freq_mean = mean(so_freq);
            so_freq_sd = std(so_freq);


            % Amplitude
            if numel(peaks)==numel(mins)
                amp = peaks-mins;
            elseif numel(peaks)>numel(mins)
                amp = peaks(2:end) - mins;
            else
                amp = peaks - mins(2:end);
            end
            
            amp_mean = mean(amp);
            amp_sd = std(amp);
            
            if plots
                %%
                figure(3)
                plot(x,time_cx_mean, locs_peaks, peaks+1,'vr', locs_mins, mins-1, '^k');
            end
            
            %-----------------------------------------
            %% UP/DOWN state detection
            % ---------------------------------------
            
            %% Detection of UP and DOWN states
            % See Volgushev (2011) doi: 10.1016/B978-0-444-53839-0.00012-0
            
            % % Histogram of voltages, should be bimodal in SO
            % v_binwidth = 1;
            % [vh_vals, vh_edges] = histcounts(time_cx_nml, 'BinWidth', v_binwidth,'Normalization', 'Probability');
            % vh_range = vh_edges < -20; % remove positive peak (spikes)
            % vh_vals = vh_vals(vh_range);
            % vh_edges = vh_edges(vh_range);
            
            % % Identify histogram peaks
            % vh_peak_ind=islocalmax(vh_vals, 'MaxNumExtrema', 2);
            
            % if numel(find(vh_peak_ind))~=2
            %     disp('Not bimodal! Do something!');
            % end
            
            % % Find the centers of the bins that islocalmax identified as peaks
            % vh_peak_left = vh_edges(vh_peak_ind);
            % vh_peak_ctr = (vh_peak_left + v_binwidth/2);
            
            % % Divide the space in between in three parts
            % vh_peak_3rd = diff(vh_peak_ctr)/3;
            
            % thr_down = vh_peak_ctr(1)+vh_peak_3rd;
            % thr_up = vh_peak_ctr(2)-vh_peak_3rd;
            
            %% Detection of UP and DOWN states
            % See Komarov (2017) DOI 10.1007/s10827-017-0663-7
            thr_down = -68;
            thr_up = -65;

            %% Create thresholded binary sequences
            t_cx_bin_up = time_cx > thr_up;
            t_cx_bin_down = time_cx < thr_down;
            
            % Clean binary activity (remove short gaps and short up or down states)
            min_ln = 100; % min length of up (down) states and gaps
            
            disp('Cleaning upstates...');tic;
            t_cx_bin_up_cl = clean_onoff(t_cx_bin_up, min_ln);
            toc;disp('done')
            disp('Cleaning downstates...');tic;
            t_cx_bin_down_cl = clean_onoff(t_cx_bin_down, min_ln);
            toc;disp('done')
            
            if plots
                figure;subplot(211);imagesc(t_cx_bin_up);subplot(212);imagesc(t_cx_bin_up_cl);
            end
            
            % %% Peaks for Ups and Downs
            % sum_up = sum(t_cx_bin_up_cl,1);
            % up_prom = (max(sum_up)-min(sum_up))/3; %prominence
            % [pk_up_bool,
            %     pk_up_prom] = islocalmax(sum_up,...
            %     'MinProminence', up_prom, 'MinSeparation', pksep);
            % pk_up = time_cx_mean(pk_up_bool);
            % locs_pk_up = find(pk_up_bool);
            % 
            % sum_down = sum(t_cx_bin_down_cl,1);
            % down_prom = (max(sum_down)-min(sum_down))/3; %prominence
            % [pk_down_bool, pk_down_prom] = islocalmax(sum_down,...
            %     'MinProminence', down_prom, 'MinSeparation', pksep);
            % pk_down = time_cx_mean(pk_down_bool);
            % locs_pk_down = find(pk_down_bool);
            % %%
            % 
            % if plots
            %     %%
            %     figure
            %     plot(x,sum_up, x, sum_down, locs_pk_up, pk_up+1,'vr', locs_pk_down, pk_down+1, '^k');
            % end

            %-----------------------------------------
            %% Onset/offset
            % ---------------------------------------
            
            %% Down-to-up times
            
            %nmins = numel(mins);
            if nmins<3
                disp(['Need at least three minima. Currently ' num2str(nmins)])
            end
            onset_diffs = nan(n,nmins-1); % 10242 x num. upstates
            offset_diffs = nan(n,nmins-1);

            disp(['Size onset_diffs ' num2str(size(onset_diffs))]);
            
            for j=1:nmins-1
                %%

                tw0=max(locs_mins(j)-min_ln, 1);
                tw1=locs_mins(j+1);

                wave_up = t_cx_bin_up_cl(:,tw0:tw1);
                wave_dn = t_cx_bin_down_cl(:,tw0:tw1);

                [diff_on, diff_off] = onoffdiff(wave_up, wave_dn, min_ln); % has the size of idx_nml

                onset_diffs(:,j) = diff_on;
                offset_diffs(:,j) = diff_off;

            end
            
            onset_diffs_sd = std(reshape(onset_diffs, 1, []), 'omitnan');
            offset_diffs_sd = std(reshape(offset_diffs, 1,[]),'omitnan');

            %save(['onset_diffs_' folder '.mat'],'onset_diffs');
            %
            if plots
                figure
                h=histogram(onset_diffs(~isnan(onset_diffs)));
                h.Normalization = 'probability';
                h.BinWidth = 50;
            end

            %% Save variables
            if isfile(['./files/so_prop/so_' folder  '.mat'])
            save(['./files/so_prop/so_' folder  '.mat'], ...
                'time_cx_mean',...
                'locs_peaks', 'peaks', 'locs_mins', 'mins',...
                'interpeak', 'interpeak_mean', 'interpeak_sd',...
                'so_freq_direct', 'so_freq', 'so_freq_mean', 'so_freq_sd',...
                'amp', 'amp_mean', 'amp_sd',...
                'thr_down','thr_up',...
                'onset_diffs', 'offset_diffs',...
                'onset_diffs_sd', 'offset_diffs_sd',"-append");
            else
                save(['./files/so_prop/so_' folder  '.mat'], ...
                'time_cx_mean',...
                'locs_peaks', 'peaks', 'locs_mins', 'mins',...
                'interpeak', 'interpeak_mean', 'interpeak_sd',...
                'so_freq_direct', 'so_freq', 'so_freq_mean', 'so_freq_sd',...
                'amp', 'amp_mean', 'amp_sd',...
                'thr_down','thr_up',...
                'onset_diffs', 'offset_diffs',...
                'onset_diffs_sd', 'offset_diffs_sd');
            end
        end %end if ~load_soprops
    end % end if ~figsonly
    
    %---------------------------
    %% Plots
    %---------------------------
    
    disp('Plotting...');
    % Plot heatmap of voltages
    f=figure;
    f.Units = 'centimeters';
    %f.Position(3:4) = [700 750];
    f.Position(3:4) = [10 12];%[18 20];
    %axwidth = 0.65;
    xt = 0:5000:30000;
    xtl=num2str(xt'/1000);
    
    tl = tiledlayout(4,2, 'Padding', 'loose', 'TileSpacing', 'tight');
    %
    ax0=nexttile([1,2]);

    yt = 0:4000:n_nml; % yticks
    ytl = strcat(num2str(yt'/1000), 'k'); %ytick labels
    c_lims=[-80,-50];% heatmap limits
    imagesc(time_cx_nml, c_lims); colormap parula; 
    cb=colorbar('south');
    title('CX layer 2 voltages');
    %xlabel('Time (ms)')
    ylabel('Neurons')
    ax0.XAxis.Visible='off';
    set(ax0, 'YTick',yt, 'YTickLabel',ytl, 'Tickdir', 'out');% Set Tick Labels
%    set(ax, 'XTick',xt, 'XTickLabel',xtl);% Set Tick Labels
    box off
    %saveas(gcf, ['./figures/' folder '_cx_heatmap.png']);
    
    % Set scale 
    hold on;
    scale_t=5000;
    plot([0 scale_t], [n_nml n_nml], 'k','Linewidth', 1.5);
    t=text(0,1,[num2str(scale_t/1000) 's']);
    t.Parent = ax0;
    t.Position = [scale_t/4, n*1.1] ;
    
    % Plot average voltage
    %figure;
    ax1=nexttile([1,2]);
    ymax = -40;
    ymin = -83;
    
    plot(x,time_cx_mean); hold on
    plot(locs_peaks, peaks+3, 'vr', 'MarkerFaceColor','r', 'MarkerSize', 3);
    plot(locs_mins, mins-3, '^r','MarkerFaceColor','r','MarkerSize', 3);

    title('Layer average voltage');
    ylabel('Voltage (mV)')
    ylim([ymin, ymax])

    if isequal(folder,'SO_base')
        txt=['('...
                    num2str(round(so_freq_mean*1000,2)) 'Hz)'...
                    ];
    else
    txt=['('...
                    num2str(round(so_freq_mean*1000,2)) 'Hz, '...
                    num2str(round(100*amp_mean/26.3)) '% base amp.)'...
                    ];
    end
    text(0.5*max(x), ymax-2,txt, 'HorizontalAlignment', 'center', 'VerticalAlignment', 'middle','FontSize', fs);
    box off
    axis off
    % ax.XAxis.Visible='off';
    hold on;
    
    scale_t=5000;
    plot([0 scale_t], [ymin ymin], 'k','Linewidth', 1);
    t=text(0,1,[num2str(scale_t/1000) 's']);
    t.Parent = ax1;
    t.Position =[scale_t/4, ymin*1.1] ;

    scale_y=20;
    plot([0 0], [ymin ymin+scale_y], 'k','Linewidth', 1);
    t=text(0,1,[num2str(scale_y) 'mV']);
    t.Parent = ax1;
    t.Position =[-scale_t*0.9, ymin+scale_y/2] ;

    % Plot single cell voltage
    ax=nexttile([1,2]);
    ymax = -50;
    ymin = -82;
    
    c1 = 1006;
    c2 = 928;
    p=plot(x,time_cx([c1,c2],:), 'Linewidth',1 );
    %%
    %p(1).Color = [1 0 0 .5];
    p(2).Color = [[1 1 1]*.68 .8];
    %p(2).LineWidth = 2;
    %%
    title (['Single-cell voltages']);
    ylim([ymin, ymax])
    %set(ax, 'XTick',xt, 'XTickLabel',xtl);% Set Tick Labels
    axis off

    % Set scale
    hold on
    scale_t=5000;
    scale_y=20;

    plot([0 scale_t], [ymin ymin], 'k','Linewidth', 1);
    t=text(0,1,[num2str(scale_t/1000) 's']);
    t.Parent = ax;
    t.Position = [scale_t/4, ymin-scale_y/3] ;

    plot([0 0], [ymin ymin+scale_y], 'k','Linewidth', 1);
    t=text(0,1,[num2str(scale_y) 'mV']);
    t.Parent = ax;
    t.Position = [-scale_t*0.9, ymin+scale_y/2] ;

    box off
        
    % Plot histograms
    
    % Voltage histogram
    nexttile(7)
    v_binwidth = 1;
    h=histogram(time_cx_nml, 'DisplayName', 'Voltage');
    h.Normalization = 'probability';
    h.BinWidth = v_binwidth;
    ymax = max(h.Values)*1.2;
    title('Voltage histogram');
    xlabel('Voltage (mV)')
    ylabel('Probability')
    
    hold on
    % xline(thr_up, '--', sprintf([num2str(round(thr_up)) 'mV']),'LineWidth',1.5, 'FontSize', 9, 'LabelHorizontalAlignment', 'center');
    % xline(thr_down, '-.', sprintf([num2str(round(thr_down)),'mV']),'LineWidth',1.5,'FontSize', 9,'LabelHorizontalAlignment', 'center');
    
    xline(thr_up, '--', sprintf(['V+']),'LineWidth',1, 'FontSize', 9, 'LabelHorizontalAlignment', 'center', 'LabelVerticalAlignment', 'middle');
    xline(thr_down, '-.', sprintf(['V-']),'LineWidth',1,'FontSize', 9,'LabelHorizontalAlignment', 'center', 'LabelVerticalAlignment', 'top');
   
    axis([-83,-50, 0, ymax]);
    box off
    
    ax=gca;
    %ax.Position(3)=axwidth*.45;
    
    % Plot onset histogram
    ms_to_s=0.001; % convert onsets to seconds

    nexttile(8)
    hold on;
    h1=histogram(onset_diffs(~isnan(onset_diffs))*ms_to_s);
    h2=histogram(offset_diffs(~isnan(offset_diffs))*ms_to_s);
    
    % Set properties
    binwidth=100*ms_to_s;

    title('Onsets/offsets');
    xlabel('Time (s)');
    box off

    set([h1,h2], 'BinWidth', binwidth, 'Normalization', 'probability', 'DisplayStyle','bar','EdgeColor', 'none');
    h1.FaceColor= [0.3010 0.7450 0.9330];
    h2.FaceColor = [0.6350 0.0780 0.1840] ;
    % Draw outline
    h21 = copyobj(h2, gca);
    h11 = copyobj(h1,gca);
    set([h11,h21],'DisplayStyle', 'stairs', 'Linewidth', 2);
    h11.EdgeColor = h1.FaceColor;
    h21.EdgeColor = h2.FaceColor;

    legend
    lgd=legend({'On','Off'},'location','northeast');
    lgd.Position = [.8 .15 .1 .08];
    legend boxoff
    set(ax, 'TickDir', 'out');
    
    % Set axis lims
    %[xmin, xmax] = bounds([onset_diffs' offset_diffs']);
    ymax = max([h1.Values h2.Values])*1.2;
    % axislims = [xmin*1.1 xmax*1.1 0 ymax];
    if ~isnan(ymax)
        axislims = [0 4 0 ymax];
        axis(axislims);
    end
    ax2=gca;
    %ax2.Position([1,3])=[ax.Position(1)+axwidth*0.55, axwidth*.45];
    
    % Fix colorbar now (it breaks with new tiles if you do it before)
    cb.Position = [ax0.Position(1)+ax0.Position(3)-.12 ax0.Position(2)-.017 .12 .015];
    set(cb,'XTick', c_lims, 'XTickLabel', strcat(num2str(c_lims'), 'mV'), 'TickDirection', 'out'); cb.Ruler.TickLabelRotation= 0;
    cb.XAxisLocation='bottom';
    %% Save fig
    %saveas(gcf, ['./figures/so_prop/so_' folder '.png']);

    mysavefig(f, './figures/so_prop_noamp_v2',['so_' folder '_noamp'],fs);

    disp(['done with folder ' folder '! (' num2str(i_fld) '/' num2str(n_fld) ')']);
    
end % end folders loop

